import logging
import os

from pg_events.core.base import Command

log = logging.getLogger(__name__)


class Build(Command):

    def execute(self):
        conn = self.establish_connection()
        conn.autocommit = True

        path = os.path.dirname(__file__)
        cursor = conn.cursor()

        log.info('Creating triggers ...')
        schema_path = os.path.join(path, 'schemas/triggers.sql')
        cursor.execute(open(schema_path, 'r').read())

        if self.auto_rebuild:
            schema_path = os.path.join(path, 'schemas/event_triggers.sql')
            cursor.execute(open(schema_path, 'r').read())

        cursor.execute('SELECT pgevents_create_db_triggers();')
        log.info('Creating triggers done.')




"""Entry-point for the :program:`pg_events` umbrella command."""
import argparse
import logging

from pg_events.core import commands

__all__ = ['main']


def main():
    logging.basicConfig(level=logging.INFO)

    parser = argparse.ArgumentParser(prog='pg_events')
    subparsers = parser.add_subparsers()

    commands.add_commands(subparsers)

    args = parser.parse_args()
    args.func(args)


if __name__ == '__main__':  # pragma: no cover
    main()



import logging
import select

from pg_events.core.base import Command

log = logging.getLogger(__name__)


DATA_UPDATE_CHANNELS = ['pgevents_data_update']
TABLE_UPDATE_CHANNELS = ['pgevents_table_alter', 'pgevents_table_drop']


class Worker(Command):

    def execute(self):
        conn = self.establish_connection()

        curs = conn.cursor()
        for channel in DATA_UPDATE_CHANNELS + TABLE_UPDATE_CHANNELS:
            curs.execute('LISTEN {};'.format(channel))

        while 1:
            if select.select([conn], [], [], 5) == ([], [], []):
                log.info('Notification Timeout')
            else:
                conn.poll()
                while conn.notifies:
                    notify = conn.notifies.pop(0)
                    log.info('Notification {} received'.format(notify.channel))
                    if notify.channel in DATA_UPDATE_CHANNELS:
                        self.data_update_callback(notify.payload)
                    elif notify.channel in TABLE_UPDATE_CHANNELS and self.auto_rebuild:
                        self.db_schema_update_callback(notify.payload)



import logging
import psycopg2
import psycopg2.extensions
import re
import sys

from pg_events.core.utils import load_module

log = logging.getLogger(__name__)


REQUIRED_SETTINGS_ATTRIBUTES = [
    'PG_EVENTS_DATABASE_URL',
    'PG_EVENTS_DATA_UPDATE_CALLBACK',
]

REQUIRED_AUTO_REBUILD_ATTRIBUTES = [
    'PG_EVENTS_DB_SCHEMA_UPDATE_CALLBACK',
]


class PgEventWorkerValidationError(Exception):
    pass


class Command(object):

    def __init__(self, args):
        if not args.settings:
            log.info("Please pass in settings module using --settings")
            sys.exit(1)

        module = re.sub('.py$', '', args.settings)
        settings = load_module(module)

        self.auto_rebuild = args.auto_rebuild

        self.validate_settings(settings)
        self.database_url = settings.PG_EVENTS_DATABASE_URL

        self.data_update_callback = self._get_function(settings.PG_EVENTS_DATA_UPDATE_CALLBACK)

        if self.auto_rebuild:
            self.db_schema_update_callback = self._get_function(settings.PG_EVENTS_DB_SCHEMA_UPDATE_CALLBACK)

    def validate_settings(self, settings):
        for attribute in REQUIRED_SETTINGS_ATTRIBUTES:
            if not hasattr(settings, attribute):
                raise PgEventWorkerValidationError('Settings should include attribute {}'.format(attribute))

        if not self.auto_rebuild:
            return

        for attribute in REQUIRED_AUTO_REBUILD_ATTRIBUTES:
            if not hasattr(settings, attribute):
                raise PgEventWorkerValidationError('Settings should include attribute {}'.format(attribute))

    def establish_connection(self):
        conn = psycopg2.connect(self.database_url)
        conn.set_isolation_level(psycopg2.extensions.ISOLATION_LEVEL_AUTOCOMMIT)

        return conn

    def _get_function(self, path):
        module, method = path.rsplit('.', 1)
        return getattr(load_module(module), method)



import importlib
import sys
import os
from contextlib import contextmanager
import imp as _imp


class NotAPackage(Exception):
    pass


@contextmanager
def cwd_in_path():
    """Context adding the current working directory to sys.path."""
    cwd = os.getcwd()
    if cwd in sys.path:
        yield
    else:
        sys.path.insert(0, cwd)
        try:
            yield cwd
        finally:
            try:
                sys.path.remove(cwd)
            except ValueError:  # pragma: no cover
                pass


def find_module(module, path=None, imp=None):
    """supporting dots"""
    if imp is None:
        imp = importlib.import_module
    with cwd_in_path():
        if '.' in module:
            last = None
            parts = module.split('.')
            for i, part in enumerate(parts[:-1]):
                mpart = imp('.'.join(parts[:i + 1]))
                try:
                    path = mpart.__path__
                except AttributeError:
                    raise NotAPackage(module)
                last = _imp.find_module(parts[i + 1], path)
            return last
        return _imp.find_module(module)


def load_module(module):
    res = None
    fp, pathname, description = find_module(module)
    try:
        res = _imp.load_module(module, fp, pathname, description)
    finally:
        if fp:
            fp.close()

    return res



import os
import setuptools

from setuptools import setup


setup(
    name='pg_events',
    version='0.1.6',
    description="Postgres events",
    long_description='',
    keywords='postgres python django trigger listen notify events heroku connect',
    author='ZeroCater',
    author_email='tech@zerocater.com',
    url='https://github.com/ZeroCater/pg_events',
    download_url='https://github.com/ZeroCater/pg_events/tarball/0.1.6',
    license='MIT',
    packages=setuptools.find_packages(),
    package_data={'pg_events': ['core/schemas/*.sql']},
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python",
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
    ],
    install_requires=[
        'psycopg2>=2.6,<2.8'
    ],
    entry_points={
        'console_scripts': [
            'pg_events = pg_events.__main__:main',
        ],
    },
)



import sys

from distutils.util import strtobool

from pg_events.core.build import Build
from pg_events.core.worker import Worker


def worker(args):
    worker = Worker(args)
    worker.execute()


def build(args):
    build = Build(args)
    build.execute()


def add_commands(subparsers):
    attach_parser = subparsers.add_parser('worker')
    attach_parser.add_argument('--settings', required=True)
    attach_parser.add_argument('--auto-rebuild', default=True,
                               type=lambda x: bool(strtobool(x)), dest='auto_rebuild')
    attach_parser.set_defaults(func=worker)

    attach_parser = subparsers.add_parser('build')
    attach_parser.add_argument('--settings', required=True)
    attach_parser.add_argument('--auto-rebuild', default=True,
                               type=lambda x: bool(strtobool(x)), dest='auto_rebuild')
    attach_parser.set_defaults(func=build)
